import { metaData } from "@cornerstonejs/core";
import cornerstoneDICOMImageLoader from "@cornerstonejs/dicom-image-loader";

/**
 * Preloads imageIds metadata in memory
 **/
async function prefetchMetadataInformation(
  imageIdsToPrefetch: string[]
): Promise<void> {
  for (let i = 0; i < imageIdsToPrefetch.length; i++) {
    await cornerstoneDICOMImageLoader.wadouri.loadImage(imageIdsToPrefetch[i])
      .promise;
  }
}

interface FrameInformation {
  frameIndex: number;
  imageIdFrameless: string;
}

function getFrameInformation(imageId: string): FrameInformation {
  if (imageId.includes("wadors:")) {
    const frameIndex = imageId.indexOf("/frames/");
    const imageIdFrameless =
      frameIndex > 0 ? imageId.slice(0, frameIndex + 8) : imageId;
    return {
      frameIndex,
      imageIdFrameless,
    };
  } else {
    const frameIndex = imageId.indexOf("&frame=");
    let imageIdFrameless =
      frameIndex > 0 ? imageId.slice(0, frameIndex + 7) : imageId;
    if (!imageIdFrameless.includes("&frame=")) {
      imageIdFrameless += "&frame=";
    }
    return {
      frameIndex,
      imageIdFrameless,
    };
  }
}

/**
 * Converts a list of imageids possibly referring to multiframe dicom images
 * into a list of imageids where each imageid refers to one frame.
 * For each imageId representing a multiframe image with n frames,
 * it creates n new imageids, one for each frame, and returns the new list of imageids.
 * If a particular imageid does not refer to a multiframe image data, it will be just copied into the new list.
 * @returns new list of imageids where each imageid represents a frame
 */
function convertMultiframeImageIds(imageIds: string[]): string[] {
  const newImageIds: string[] = [];
  imageIds.forEach((imageId) => {
    const { imageIdFrameless } = getFrameInformation(imageId);
    const instanceMetaData = metaData.get("multiframeModule", imageId);
    if (
      instanceMetaData &&
      instanceMetaData.NumberOfFrames &&
      instanceMetaData.NumberOfFrames > 1
    ) {
      const numberOfFrames = instanceMetaData.NumberOfFrames;
      for (let i = 0; i < numberOfFrames; i++) {
        const newImageId = imageIdFrameless + (i + 1);
        newImageIds.push(newImageId);
      }
    } else {
      newImageIds.push(imageId);
    }
  });
  return newImageIds;
}

export { convertMultiframeImageIds, prefetchMetadataInformation };
